#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gdbm.h>

#include "agenda.h"
#include "lista.h"


static GDBM_FILE dbf = NULL;


int db_open(char *db_nome, int mode)
{
  int stat = 0;

  if (mode == UPDATE)
    mode = GDBM_WRCREAT;
  else  if (mode == QUERY)
    mode = GDBM_READER;

  if (dbf == NULL)
    dbf = gdbm_open (db_nome, 512, mode, 0644, 0);

  if (dbf)
    stat = 1;
  else
    fprintf(stderr, "Erro ao criar base de dados %s\n", DBASE_NOME);

  return stat;
    
}



void db_close()
{
  gdbm_close(dbf);
  dbf = NULL;
}



/* Grava agenda em arquivo */
int db_store (lista *agenda)
{
  int status = 0, i;
  datum key, content;
  char chave[TAM_NOME + TAM_FONE] = {'\0'};

  while (agenda) {
    strcat (chave, agenda->item->nome);
    key.dptr = strcat (chave, agenda->item->fone);
    key.dsize = strlen(key.dptr) + 1;
    content.dptr = (char *) agenda->item;
    content.dsize = sizeof(endereco);
    status = gdbm_store (dbf, key, content, GDBM_REPLACE);

    agenda = agenda->next;
  }

  return !status;
}



/* Le itens da agenda de arquivo */
static lista *db_readDB ()
{
  lista *agenda = NULL;
  endereco *item;
  datum key, content;

  key = gdbm_firstkey (dbf);

  while (key.dptr) {
    item = (endereco *) calloc (1, sizeof (endereco));
    if (!item) {
      fprintf (stderr, "Erro de alocacao de memoria em db_readLista()@agdbm.c.\n");
      exit (2);
    }
    else {
      content = gdbm_fetch (dbf, key);
      memcpy (item, content.dptr, content.dsize);
      agenda = insere (item, agenda);
      key = gdbm_nextkey (dbf, key);
    }
  }

  return agenda;
}


lista *db_query (char *key, int query_type)
{
  lista *agenda,
        *query = NULL;

  agenda = db_readDB();

  while (agenda) {
    switch (query_type) {
    case QUERY_BY_NAME:
      if (strstr (agenda->item->nome, key))
	query = insere (agenda->item, query);
      break;
    case QUERY_BY_EMAIL:
      if (strstr (agenda->item->email, key))
	query = insere (agenda->item, query);
      break;
    case QUERY_BY_DATE:
      if (strstr (agenda->item->aniversario, key))
	query = insere (agenda->item, query);
      break;
    case QUERY_BY_DAYS:
      if (strstr (agenda->item->email, key))  /* precisa alterar */
	query = insere (agenda->item, query);
      break;
    }

    agenda = agenda->next;
  }

  return query;
}
